package ru.skipy.tests.ui;

import ru.skipy.tests.ui.delegate.ComponentContainer;
import ru.skipy.tests.ui.delegate.ComponentRendererFactory;

import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import java.awt.*;
import java.util.Vector;

/**
 * PaintDelegationTest
 * <p/>
 * Paint delegation sample
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 20.12.2010
 */
public class PaintDelegationTest extends JFrame {

    /**
     * Component types
     */
    private JComboBox types;

    /**
     * Creates main UI frame
     */
    public PaintDelegationTest() {
        super("Paint delegation test");
        JPanel cp = new JPanel(new BorderLayout(5, 5));
        cp.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(5, 5, 5, 5),
                BorderFactory.createLineBorder(Color.red)));
        setContentPane(cp);
        ComponentContainer cc = new ComponentContainer();
        cc.setRendererFactory(new ComponentRendererFactoryImpl());
        cp.add(cc, BorderLayout.CENTER);
        types = new JComboBox(new Object[]{new JButtonComponentDescriptor(),
                new JCheckBoxComponentDescriptor(true),
                new JCheckBoxComponentDescriptor(false),
                new JTextFieldComponentDescriptor(),
                new JTreeComponentDescriptor(),
                new JTableComponentDescriptor()
        });
        cp.add(types, BorderLayout.SOUTH);
        types.setSelectedIndex(0);
        setSize(800, 600);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
    }

    public static void main(String[] args) {
        new PaintDelegationTest().setVisible(true);
    }

    /**
     * Implementation of {@link ComponentRendererFactory}
     */
    class ComponentRendererFactoryImpl implements ComponentRendererFactory {

        public JComponent createRenderer() {
            Object o = types.getSelectedItem();
            if (!(o instanceof ComponentDescriptor))
                return null;
            return ((ComponentDescriptor) o).createComponent();
        }
    }

    /**
     * Component descriptor and actually renderer factory
     */
    private abstract class ComponentDescriptor {

        /**
         * title to show in the component types list
         */
        private String title;

        /**
         * created component counter
         */
        protected int counter;

        protected ComponentDescriptor(String title) {
            this.title = title;
        }

        @Override
        public String toString() {
            return title;
        }

        /**
         * Creates renderer
         *
         * @return renderer component
         */
        abstract JComponent createComponent();
    }

    /**
     * JButton renderer descriptor
     */
    private class JButtonComponentDescriptor extends ComponentDescriptor {

        JButtonComponentDescriptor() {
            super("JButton component");
        }

        @Override
        JComponent createComponent() {
            return new JButton("Button #" + counter++);
        }
    }

    /**
     * JCheckBox renderer descriptor
     */
    private class JCheckBoxComponentDescriptor extends ComponentDescriptor {

        private boolean selected;

        JCheckBoxComponentDescriptor(boolean selected) {
            super("JCheckBox component " + (selected ? "(selected)" : "(unselected)"));
            this.selected = selected;
        }

        @Override
        JComponent createComponent() {
            return new JCheckBox("CheckBox #" + counter++, selected);
        }
    }

    /**
     * JtextField renderer descriptor
     */
    private class JTextFieldComponentDescriptor extends ComponentDescriptor {

        JTextFieldComponentDescriptor() {
            super("JTextField component");
        }

        @Override
        JComponent createComponent() {
            return new JTextField("Text field #" + counter++);
        }
    }

    /**
     * JTree renderer descriptor
     */
    private class JTreeComponentDescriptor extends ComponentDescriptor {

        JTreeComponentDescriptor() {
            super("JTree component");
        }

        @Override
        JComponent createComponent() {
            Vector<Object> data = new Vector<Object>();
            data.add("Leaf 1");
            Vector<Object> node = new Vector<Object>();
            node.add("Leaf 2");
            node.add("Leaf 3");
            node.add("Leaf 4");
            data.add(node);
            JTree tree = new JTree(data);
            tree.expandRow(1);
            tree.setBorder(BorderFactory.createLineBorder(Color.lightGray));
            return tree;
        }
    }

    /**
     * JTable renderer descriptor
     */
    private class JTableComponentDescriptor extends ComponentDescriptor {

        JTableComponentDescriptor() {
            super("JTable component");
        }

        @Override
        JComponent createComponent() {
            TableModel m = new TableModel() {
                private String columns[] = {"ID", "Name", "Active"};
                private Class classes[] = {Integer.class, String.class, Boolean.class};
                private Object data[][] = {{1001, "Administrator", true}, {1002, "Skipy", true}, {1003, "John Doe", false}};

                public int getRowCount() {
                    return 3;
                }

                public int getColumnCount() {
                    return 3;
                }

                public String getColumnName(int columnIndex) {
                    return columns[columnIndex];
                }

                public Class<?> getColumnClass(int columnIndex) {
                    return classes[columnIndex];
                }

                public boolean isCellEditable(int rowIndex, int columnIndex) {
                    return false;
                }

                public Object getValueAt(int rowIndex, int columnIndex) {
                    return data[rowIndex][columnIndex];
                }

                public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
                }

                public void addTableModelListener(TableModelListener l) {
                }

                public void removeTableModelListener(TableModelListener l) {
                }
            };
            JTable table = new JTable(m) {
            };
            table.setBorder(BorderFactory.createLineBorder(Color.lightGray));
            table.getColumnModel().getColumn(1).setMinWidth(100);
            return table;
        }
    }


}
