package ru.skipy.tests.ui.lnf;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.metal.MetalTextFieldUI;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * HighlightedTextFieldUI
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since Sep 22, 2010
 */
public class HighlightedTextFieldUI extends MetalTextFieldUI {

    /**
     * Events handler
     */
    private final Highlighter highlighter = new Highlighter();

    /**
     * Sets this delegate to component
     *
     * @param c component to set delegate
     */
    @Override
    public void installUI(JComponent c) {
        super.installUI(c);
        if (c instanceof JTextField) {
            c.addFocusListener(highlighter);
            c.addMouseListener(highlighter);
        }
    }

    /**
     * Removes this delegate from component
     *
     * @param c component to remove delegate from
     */
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        if (c instanceof JTextField) {
            c.removeFocusListener(highlighter);
            c.removeMouseListener(highlighter);
        }
    }

    /**
     * Creates new delegate instance
     *
     * @param c component
     * @return delegate instance
     */
    @SuppressWarnings("UnusedDeclaration")
    public static ComponentUI createUI(JComponent c) {
        return new HighlightedTextFieldUI();
    }

    /**
     * Mouse events handler
     */
    public static class Highlighter extends MouseAdapter implements FocusListener {

        /**
         * Key to store border object
         */
        public static final String KEY_UNFOCUSED_BORDER = "unfocused-border";

        /**
         * Key to store background color object
         */
        public static final String KEY_UNHIGHLIGHTED_BG = "unhighlighted-bg";

        /**
         * Highlighted border
         */
        public static final Border HIGHLIGHTED_BORDER =
                BorderFactory.createLineBorder(new Color(255, 96, 0), 1);

        /**
         * Highlighted background color
         */
        public static final Color HIGHLIGHTED_BG = new Color(176, 208, 255);

        /**
         * Focus gain event handler
         *
         * @param e event
         */
        @Override
        public void focusGained(FocusEvent e) {
            if (!(e.getComponent() instanceof JTextField))
                return;
            JTextField tf = (JTextField) e.getComponent();
            Border currentBorder = tf.getBorder();
            tf.putClientProperty(KEY_UNFOCUSED_BORDER, currentBorder);
            tf.setBorder(HIGHLIGHTED_BORDER);
        }

        /**
         * Focus lost event handler
         *
         * @param e event
         */
        @Override
        public void focusLost(FocusEvent e) {
            if (!(e.getComponent() instanceof JTextField))
                return;
            JTextField tf = (JTextField) e.getComponent();
            Object property = tf.getClientProperty(KEY_UNFOCUSED_BORDER);
            if (property instanceof Border) {
                tf.setBorder((Border) property);
            } else {
                tf.setBorder(UIManager.getBorder("TextField.border"));
            }
        }

        /**
         * Mouse entered event handler
         *
         * @param e event
         */
        @Override
        public void mouseEntered(MouseEvent e) {
            if (!(e.getComponent() instanceof JTextField))
                return;
            JTextField tf = (JTextField) e.getComponent();
            Color currentBG = tf.getBackground();
            tf.putClientProperty(KEY_UNHIGHLIGHTED_BG, currentBG);
            tf.setBackground(HIGHLIGHTED_BG);
        }

        /**
         * Mouse exited event handler
         *
         * @param e event
         */
        @Override
        public void mouseExited(MouseEvent e) {
            if (!(e.getComponent() instanceof JTextField))
                return;
            JTextField tf = (JTextField) e.getComponent();
            Object property = tf.getClientProperty(KEY_UNHIGHLIGHTED_BG);
            if (property instanceof Color) {
                tf.setBackground((Color) property);
            } else {
                tf.setBackground(UIManager.getColor("TextField.background"));
            }
        }
    }
}
