package ru.skipy.tests.ui.mouse_paints;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

/**
 * FreeMousePaintTest - Sample application to demonstrate mouse painting techniques
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since Sep 21, 2010
 */
public class FreeMousePaintTest extends JFrame {

    FreeMousePaintTest(BorderType bt, boolean checkBorder) {
        super("Free Mouse Paint test");
        DrawPanel dp = new DrawPanel(checkBorder);
        dp.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.blue),
                BorderFactory.createCompoundBorder(bt.getBorder(),
                        BorderFactory.createLineBorder(Color.blue))));
        JPanel cp = new JPanel(new BorderLayout());
        cp.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        cp.setBackground(Color.white);
        setContentPane(cp);
        cp.add(dp);
        setSize(800, 600);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLocationRelativeTo(null);
    }

    public static void main(String[] args) {
        String borderType = System.getProperty("borderType");
        BorderType bt = BorderType.empty;
        if (borderType != null && borderType.length() > 0)
            try {
                bt = BorderType.valueOf(borderType);
            } catch (IllegalArgumentException ex) {
                // do nothing
            }
        String checkBorder = System.getProperty("checkBorder");
        new FreeMousePaintTest(bt, Boolean.parseBoolean(checkBorder)).setVisible(true);
    }

    /**
     * Drawing panel
     */
    static class DrawPanel extends JPanel {

        /**
         * Color for mouse track
         */
        private Color mid = new Color(0, 192, 0);

        /**
         * Buffer minimal width
         */
        private int minW = -1;
        /**
         * Buffer minimal height
         */
        private int minH = -1;

        /**
         * Background buffer
         */
        private BufferedImage buffer = null;

        /**
         * Whether we should take border size into account
         */
        boolean checkBorder;

        DrawPanel(boolean checkBorder) {
            addComponentListener(new ComponentListenerImpl());
            MouseHandler handler = new MouseHandler();
            addMouseListener(handler);
            addMouseMotionListener(handler);
            setBackground(new Color(255, 255, 160));
            this.checkBorder = checkBorder;
        }

        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            if (buffer != null) {
                Border b = getBorder();
                int x = 0;
                int y = 0;
                Shape oldClip = g.getClip();
                if (checkBorder && b != null) {
                    Insets i = b.getBorderInsets(DrawPanel.this);
                    x += i.left;
                    y += i.top;
                    Rectangle2D.Float clip = new Rectangle2D.Float(x, y,
                            getWidth() - i.left - i.right, getHeight() - i.top - i.bottom);
                    g.setClip(clip);
                }
                g.drawImage(buffer, x, y, this);
                g.setClip(oldClip);
            }
        }

        /**
         * Mouse handler to handle paint actiona
         */
        private class MouseHandler extends MouseAdapter {
            @Override
            public void mouseDragged(MouseEvent e) {
                Border b = getBorder();
                int x = e.getX();
                int y = e.getY();
                if (checkBorder && b != null) {
                    Insets i = b.getBorderInsets(DrawPanel.this);
                    x -= i.left;
                    y -= i.top;
                }
                draw(x, y, Color.gray, false);
            }

            @Override
            public void mousePressed(MouseEvent e) {
                Border b = getBorder();
                int x = e.getX();
                int y = e.getY();
                if (checkBorder && b != null) {
                    Insets i = b.getBorderInsets(DrawPanel.this);
                    x -= i.left;
                    y -= i.top;
                }
                draw(x, y, Color.red, true);
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                Border b = getBorder();
                int x = e.getX();
                int y = e.getY();
                if (checkBorder && b != null) {
                    Insets i = b.getBorderInsets(DrawPanel.this);
                    x -= i.left;
                    y -= i.top;
                }
                draw(x, y, mid, true);
            }

            /**
             * Draws mark on the buffer and repaints whole panel
             *
             * @param x     x-coordinate of mouse
             * @param y     y-coordinate of mouse
             * @param color mark color
             * @param fill  whether to fill mark
             */
            private void draw(int x, int y, Color color, boolean fill) {
                if (buffer == null) return;
                Graphics2D g = buffer.createGraphics();
                g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                g.setColor(color);
                if (fill)
                    g.fillRect(x - 2, y - 2, 4, 4);
                else
                    g.drawRect(x - 2, y - 2, 4, 4);
                DrawPanel.this.repaint();
            }
        }

        /**
         * Component events handler to track necessity to change buffer size
         */
        private class ComponentListenerImpl extends ComponentAdapter {
            @Override
            public void componentResized(ComponentEvent e) {
                adjustSizes();
            }

            @Override
            public void componentShown(ComponentEvent e) {
                adjustSizes();
            }

            /**
             * Size adjustment method
             */
            private void adjustSizes() {
                boolean adjustNeeded = false;
                int width = getWidth();
                int height = getHeight();
                Border b = getBorder();
                if (checkBorder && b != null) {
                    Insets i = b.getBorderInsets(DrawPanel.this);
                    width -= (i.left + i.right);
                    height -= (i.top + i.bottom);
                }

                if (width > minW) {
                    minW = width;
                    adjustNeeded = true;
                }
                if (height > minH) {
                    minH = height;
                    adjustNeeded = true;
                }
                if (adjustNeeded) {
                    BufferedImage newBuffer = new BufferedImage(minW, minH, BufferedImage.TYPE_INT_ARGB);
                    Graphics g = newBuffer.createGraphics();
                    g.setColor(getBackground());
                    g.fillRect(0, 0, minW, minH);
                    if (buffer != null) {
                        newBuffer.getGraphics().drawImage(buffer, 0, 0, DrawPanel.this);
                    }
                    buffer = newBuffer;
                }
            }
        }
    }

    /**
     * border factories
     */
    @SuppressWarnings("UnusedDeclaration")
    static enum BorderType {

        colored {
            @Override
            Border getBorder() {
                return BorderFactory.createLineBorder(new Color(255, 128, 255, 128), 15);
            }},
        empty {
            @Override
            Border getBorder() {
                return BorderFactory.createEmptyBorder(15, 15, 15, 15);
            }};

        abstract Border getBorder();

    }

}
