package ru.skipy.tests.ui.mouse_paints;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Random;

/**
 * XorPaintTest - sample application to demonstrate XOR painting techniques
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 13.10.2010
 */
public class XorPaintTest extends JFrame {

    public XorPaintTest() {
        super("XOR paint test");
        JPanel cp = new JPanel(new BorderLayout(5, 5));
        cp.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        final DrawPanel dp = new DrawPanel();
        dp.setBorder(BorderFactory.createLineBorder(Color.blue));
        cp.add(dp, BorderLayout.CENTER);
        final JCheckBox cb = new JCheckBox("Emulate asynchronous redrawing");
        cb.setSelected(false);
        cb.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dp.setUnexpectedRepaints(cb.isSelected());
            }
        });
        cp.add(cb, BorderLayout.SOUTH);
        setContentPane(cp);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setSize(800, 600);
        setLocationRelativeTo(null);
    }

    public static void main(String[] args) {
        new XorPaintTest().setVisible(true);
    }

    /**
     * Drawing panel with XOR-drawing capability and unexpected paints emulation
     */
    static class DrawPanel extends JPanel {

        private static final Color xorColor = Color.cyan;
        private static final Color paintColor = Color.blue;

        /**
         * Painted figures enclosing rectangles collection
         */
        private java.util.List<Rectangle> rects;

        /**
         * repaints emulation flag
         */
        private volatile boolean unexpectedRepaints = false;

        DrawPanel() {
            MouseHandler mh = new MouseHandler();
            addMouseListener(mh);
            addMouseMotionListener(mh);
            setBackground(Color.white);
            rects = new ArrayList<Rectangle>();
            Thread t = new Thread(new Runnable() {
                Random r = new Random(System.currentTimeMillis());

                @Override
                public void run() {
                    while (true) {
                        try {
                            Thread.sleep(r.nextInt(500) + 500);
                        } catch (InterruptedException ex) {
                            ex.printStackTrace();
                            return;
                        }
                        if (unexpectedRepaints) {
                            DrawPanel.this.repaint();
                        }
                    }
                }
            }, "Unexpected repaints thread");
            t.setDaemon(true);
            t.start();
        }

        public void setUnexpectedRepaints(boolean value) {
            unexpectedRepaints = value;
        }

        /**
         * Painting all figures on the panel
         *
         * @param g graphic context
         */
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            Graphics2D g2d = (Graphics2D) g;
            Paint p = g2d.getPaint();
            g2d.setColor(paintColor);
            for (Rectangle rect : rects) {
                g2d.drawOval(rect.x, rect.y, rect.width, rect.height);
            }
            g2d.setPaint(p);
        }

        /**
         * Mouse event handler to perform painting
         */
        class MouseHandler extends MouseAdapter {

            private int x1;
            private int y1;
            private int x2;
            private int y2;

            /**
             * Painting start method
             *
             * @param e mouse event
             */
            @Override
            public void mousePressed(MouseEvent e) {
                x1 = e.getX();
                y1 = e.getY();
                x2 = x1;
                y2 = y1;
                Graphics2D g2d = (Graphics2D) getGraphics();
                Paint p = g2d.getPaint();
                g2d.setXORMode(xorColor);
                g2d.drawOval(x1, y1, 0, 0);
                g2d.setPaintMode();
                g2d.setPaint(p);
            }

            /**
             * Final painting method
             *
             * @param e mouse event
             */
            @Override
            public void mouseReleased(MouseEvent e) {
                int ltX = Math.min(x1, x2);
                int ltY = Math.min(y1, y2);
                int rbX = Math.max(x1, x2);
                int rbY = Math.max(y1, y2);
                rects.add(new Rectangle(ltX, ltY, rbX - ltX, rbY - ltY));
                repaint(0);
            }

            /**
             * Painting during the movement method
             *
             * @param e mouse event
             */
            @Override
            public void mouseDragged(MouseEvent e) {
                int ltX = Math.min(x1, x2);
                int ltY = Math.min(y1, y2);
                int rbX = Math.max(x1, x2);
                int rbY = Math.max(y1, y2);
                Graphics2D g2d = (Graphics2D) getGraphics();
                Paint p = g2d.getPaint();
                // compensating previous paint
                g2d.setXORMode(xorColor);
                g2d.drawOval(ltX, ltY, rbX - ltX, rbY - ltY);
                x2 = e.getX();
                y2 = e.getY();
                ltX = Math.min(x1, x2);
                ltY = Math.min(y1, y2);
                rbX = Math.max(x1, x2);
                rbY = Math.max(y1, y2);
                // painting new figure
                g2d.drawOval(ltX, ltY, rbX - ltX, rbY - ltY);
                g2d.setPaintMode();
                g2d.setPaint(p);
            }
        }

    }

}
